import nltk
from nltk.stem.wordnet import WordNetLemmatizer
from textblob.blob import BaseBlob, Word, WordList
from textblob.compat import unicode
from textblob.decorators import requires_nltk_corpus
from textblob_ar.sentiment import PatternAnalyzer
from textblob_ar.tokenizer import WordPunctTokenizer
from textblob_ar.pos_tagger import StanfordPartOfSpeechTagger


class WordAR(Word):
    """Arabic Word representation and basic ops"""

    def translate(self, to='en'):
        """Translate the word from arabic to another language using Google's
        Translate API.
        :param to: string represents the language code for the target language. 
        """
        return Word.translate(from_lang='ar', to=to)

    def spellcheck(self):
        raise NotImplementedError

    def correct(self):
        raise NotImplementedError

    @requires_nltk_corpus
    def singularize(self):
        """Return the singular version of the word as a Word."""
        raise NotImplementedError

    @requires_nltk_corpus
    def pluralize(self):
        '''Return the plural version of the word as a Word.'''
        raise NotImplementedError

    # todo
    # word description
    # arabic diacritics


class WordlistAR(WordList):
    """Wordlist Data structure

    A list-like data structure to process words customized for arabic words. 
    """

    def count(self, strg,  *args, **kwargs):
        """Get the count of a word or phrase `s` within this WordList.
        Overrides textblob base wordlist.
        :param strg: The string to count.
        """
        return self._collection.count(strg, *args, **kwargs)

    def lemmatize(self):
        """Return the lemma of each word in this WordList."""

        return self.__class__([i.lemma() for i in self])


class TextBlob(BaseBlob):

    def __init__(self, text, tokenizer=None,
                 pos_tagger=None,
                 np_extractor=None,
                 analyzer=None,
                 parser=None,
                 classifier=None, clean_html=False):
        _tokenizer = tokenizer if tokenizer is not None else WordPunctTokenizer()
        _analyzer = analyzer if analyzer is not None else PatternAnalyzer()
        _pos_tagger = pos_tagger if pos_tagger is not None else StanfordPartOfSpeechTagger()
        super().__init__(text=text, tokenizer=_tokenizer, analyzer=_analyzer,
                         pos_tagger=_pos_tagger, clean_html=clean_html)

    def stopwords(self):
        '''Return list of arabic stopwords'''
        return nltk.corpus.stopwords.words("arabic")

    def tag(self):
        return self.pos_tagger.tag(self.tokens)



from nltk import StanfordPOSTagger
from textblob.taggers import BaseTagger
from pprint import pprint


class StanfordPartOfSpeechTagger(BaseTagger):
    """An interface for stanford arabic pos"""

    def __init__(self):
        modelPath = 'textblob_ar/data/arabic.tagger'
        modelJar = 'textblob_ar/data/model.jar'
        self._tagger = StanfordPOSTagger(modelPath, modelJar)

    def tag(self, text, tokenize=True):
        """Return a list of tuples of the form (word, tag)
        for a given set of text or BaseBlob instance.
        """

        if tokenize:
            tokens = text.tokens
        else:
            tokens = text
        tags = self._tagger.tag(tokens=tokens)
        result = []
        for tag in tags:
            if tag[0]:
                result.append(tuple(tag[1].split('/')))
            else:
                result.append(tag)
        return [i for i in result if len(i) == 2]



from __future__ import absolute_import
from textblob_ar.blob import TextBlob
from textblob_ar.similarity import TextSimilarity
from textblob_ar.correction import TextCorrection

__version__ = '0.0.2'
__author__ = 'Adham Ehab'
__license__ = "MIT"



import re
from collections import Counter


class TextCorrection:

    def __init__(self):
        wl1 = Counter(self.words(open('data/bbc-arabic-wordlist.csv').read()))
        wl2 = Counter(self.words(open('data/OSAc-wordlist.csv').read()))
        self.WORDS = wl1 + wl2

    def words(self, text):
        return re.findall(r'\w+', text.lower())

    def P(self, word): 
        "Probability of `word`."
        N = sum(self.WORDS.values())
        return self.WORDS[word] / N

    def correction(self, word, top=False):
        "Most probable spelling correction for word."
        if top:
            return max(self.candidates(word), key=self.P)
        return self.candidates(word)

    def candidates(self, word):
        "Generate possible spelling corrections for word."
        return (self.known([word]) or self.known(self.edits1(word)) or self.known(self.edits2(word)) or [word])

    def known(self, words):
        "The subset of `words` that appear in the dictionary of WORDS."
        return set(w for w in words if w in self.WORDS)

    def edits1(self, word):
        "All edits that are one edit away from `word`."
        letters    = 'غظضذخثتشرقصفعسنملكيطحزوهدجبأ'
        splits     = [(word[:i], word[i:])    for i in range(len(word) + 1)]
        deletes    = [L + R[1:]               for L, R in splits if R]
        transposes = [L + R[1] + R[0] + R[2:] for L, R in splits if len(R)>1]
        replaces   = [L + c + R[1:]           for L, R in splits if R for c in letters]
        inserts    = [L + c + R               for L, R in splits for c in letters]
        return set(deletes + transposes + replaces + inserts)

    def edits2(self, word): 
        "All edits that are two edits away from `word`."
        return (e2 for e1 in self.edits1(word) for e2 in self.edits1(e1))




from __future__ import absolute_import
from textblob.base import BaseSentimentAnalyzer, CONTINUOUS
from textblob.translate import Translator
from textblob.sentiments import PatternAnalyzer as PA


class PatternAnalyzer(BaseSentimentAnalyzer):

    '''Sentiment analyzer that uses the same implementation as the
    pattern library. Returns results as a tuple of the form:
    ``(polarity, subjectivity)``
    '''

    kind = CONTINUOUS

    def analyze(self, text):
        """Return the sentiment as a tuple of the form:
        ``(polarity, subjectivity)``

        First the text is translated into english text
        using base translator from textblob then
        the sentiment is calculated by PatternAnalyzer
        """
        translated_text = Translator().translate(text, from_lang='ar')
        return PA().analyze(translated_text)




import re


'''
Code snippet from https://github.com/bakrianoo/aravec

'''


def clean_str(text):
    search = ["أ", "إ", "آ", "ة", "_", "-", "/", ".", "،", " و ", " يا ",
              '"', "ـ", "'", "ى", "\\", '\n', '\t', '&quot;', '?', '؟', '!']
    replace = ["ا", "ا", "ا", "ه", " ", " ", "", "", "", " و",
               " يا", "", "", "", "ي", "", ' ', ' ', ' ', ' ? ', ' ؟ ', ' ! ']

    p_tashkeel = re.compile(r'[\u0617-\u061A\u064B-\u0652]')
    text = re.sub(p_tashkeel, "", text)
    p_longation = re.compile(r'(.)\1+')
    subst = r"\1\1"
    text = re.sub(p_longation, subst, text)
    text = text.replace('وو', 'و')
    text = text.replace('يي', 'ي')
    text = text.replace('اا', 'ا')

    for i in range(0, len(search)):
        text = text.replace(search[i], replace[i])
    text = text.strip()

    return text



from nltk import TreebankWordTokenizer
from nltk import WordPunctTokenizer
from nltk import WhitespaceTokenizer


from textblob.base import BaseTokenizer



        
class NLTKTreebankWordTokenizer(BaseTokenizer):

    def tokenize(self, text):

        return TreebankWordTokenizer().tokenize(text)

class NLTKWordPunctTokenizer(BaseTokenizer):

    def tokenize(self, text):

        return WordPunctTokenizer().tokenize(text)


class NLTKWhitespaceTokenizer(BaseTokenizer):

    def tokenize(self, text):
        return WhitespaceTokenizer().tokenize(text)

        


import numpy as np
import gensim
#from gensim.models.KeyedVectors import load_word2vec_format
from scipy import spatial

from textblob_ar.tokenizer import NLTKWordPunctTokenizer
from textblob_ar.utils import clean_str


class TextSimilarity:

    def __init__(self):
        try:
            self.model = gensim.models.KeyedVectors.load_word2vec_format('wiki.ar.vec')
            self.index2word_set = set(self.model.wv.index2word)
        except FileNotFoundError:
            raise FileNotFoundError
    def avg_feature_vector(self, sentence, num_features=300):
        words = NLTKWordPunctTokenizer().tokenize(clean_str(sentence))
        feature_vec = np.zeros((num_features, ), dtype='float32')
        n_words = 0
        for word in words:
            if word in self.index2word_set:
                n_words += 1
                feature_vec = np.add(feature_vec, self.model[word])
        if (n_words > 0):
            feature_vec = np.divide(feature_vec, n_words)
        return feature_vec

    def similarity(self, sentence1, sentence2):
        vec1, vec2 = self.avg_feature_vector(sentence1), self.avg_feature_vector(sentence2)
        return self.cosine_similarity(vec1, vec2)

    def cosine_similarity(self, vec1, vec2):
        return 1 - spatial.distance.cosine(vec1, vec2)